﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Media;

namespace Game
{
    public struct instanceData
    {
        public SoundEffectInstance soundInstance;
        public bool fade;
        public int vol; //de 0 a 1000
        public int maxVol; //de 0 a 1000
        public int fadeSpeed;
        public bool fadeIn;
        public bool fadeOut;
        public bool pausedByStart;
        public bool stopAfterFade;
    }

    public class AudioManager
    {
        //pruebas de clases y métodos
        bool init = true;
        int cont = 0;
        SoundEffect soundEffect;
        SoundEffectInstance soundEffectInstance;
        SoundEffectInstance soundEffectInstance2;
        Song song;
        int vol = 1000;
        //Fin pruebas.

        public SoundEffect[] soundArray = new SoundEffect[12];

        public instanceData[] instanceArray = new instanceData[1];

        public AudioManager(ContentManager Content)
        {
            SoundEffect.MasterVolume = 1;

            //Sounds

            soundArray[0] = Content.Load<SoundEffect>(@"Audio/menuBack");
            soundArray[1] = Content.Load<SoundEffect>(@"Audio/moveOption");
            soundArray[2] = Content.Load<SoundEffect>(@"Audio/selectOption");            
            soundArray[3] = Content.Load<SoundEffect>(@"Audio/shoot1");

            soundArray[4] = Content.Load<SoundEffect>(@"Audio/explosion1");
            soundArray[5] = Content.Load<SoundEffect>(@"Audio/explosion2");
            soundArray[6] = Content.Load<SoundEffect>(@"Audio/absorb1");
            soundArray[7] = Content.Load<SoundEffect>(@"Audio/absorb2");
            soundArray[8] = Content.Load<SoundEffect>(@"Audio/explosionPlayer");

            soundArray[9] = Content.Load<SoundEffect>(@"Audio/overheatAdvise");
            soundArray[10] = Content.Load<SoundEffect>(@"Audio/overheat");

            soundArray[11] = Content.Load<SoundEffect>(@"Audio/menuBack");
            
            //Sound instances
            instanceArray[0].soundInstance = Content.Load<SoundEffect>(@"Audio/virgo").CreateInstance();
            instanceArray[0].soundInstance.IsLooped = true;
            instanceArray[0].maxVol = 500;
            
            //instanceArray[1].soundInstance = contentLoader.Content.Load<SoundEffect>(@"Audio/yuuyakenagisa").CreateInstance();
            //instanceArray[1].soundInstance.IsLooped = true;
            //instanceArray[1].maxVol = 800;
           
            //pruebas de clases y métodos
            //soundEffect = contentLoader.Content.Load<SoundEffect>(@"Audio/drum-japanese1");
            //soundEffectInstance = soundEffect.CreateInstance();
            //soundEffectInstance2 = soundEffect.CreateInstance();
            //song = contentLoader.Content.Load<Song>(@"Audio/prueba");
        }

        public void playSound(int id, int vol, float pan) //id coincide con la posición del sonido en la lista.
        {
            soundArray[id].Play(vol/1000f, 0, pan);
        }

        public void playInstance(int id, int fadeSpeed)
        {
            if (instanceArray[id].fadeOut) instanceArray[id].fadeIn = true;
            instanceArray[id].fadeOut = false;
            if (instanceArray[id].soundInstance.State != SoundState.Playing)
            {
                instanceArray[id].soundInstance.Play();
                if (fadeSpeed == -1) { instanceArray[id].soundInstance.Volume = instanceArray[id].maxVol/1000f ; instanceArray[id].vol = instanceArray[id].maxVol; }
                else
                {
                    instanceArray[id].fadeIn = true;
                    instanceArray[id].vol = 0;
                    instanceArray[id].soundInstance.Volume = 0;
                    instanceArray[id].fadeSpeed = fadeSpeed;
                }
            }
        }
        public void stopInstance(int id, int fadeSpeed)
        {
            instanceArray[id].fadeIn = false;
            if (fadeSpeed == -1) instanceArray[id].soundInstance.Stop();
            else
            {
                instanceArray[id].fadeOut = true;
                instanceArray[id].fadeSpeed = fadeSpeed;
                instanceArray[id].stopAfterFade = true;
            }
        }

        public void pauseInstance(int id)
        {
            instanceArray[id].soundInstance.Pause();
        }

        public void resumeInstance(int id)
        {
            instanceArray[id].soundInstance.Resume();
        }

        public void fadeOutInstance(int id, int fadeSpeed)
        {
            instanceArray[id].fadeIn = false;
            instanceArray[id].fadeOut = true;
            instanceArray[id].fadeSpeed = fadeSpeed;
            instanceArray[id].stopAfterFade = false;
        }

        public void fadeInInstance(int id, int fadeSpeed)
        {
            instanceArray[id].fadeIn = true;
            instanceArray[id].fadeOut = false;
            instanceArray[id].fadeSpeed = fadeSpeed;
            instanceArray[id].stopAfterFade = false;
        }

        public void setVolumeInstance(int id, int vol)
        {
            
            instanceArray[id].soundInstance.Volume = vol/1000f;
        }

        public void loopInstance(int id, bool loop)
        {
            if (instanceArray[id].soundInstance.State != SoundState.Playing) instanceArray[id].soundInstance.IsLooped = loop;
        }
     
        public void update()
        {
                        
            for (int i = 0; i < instanceArray.Length; i++)
            {
                if (instanceArray[i].fadeIn)
                {
                    if (instanceArray[i].vol < instanceArray[i].maxVol)
                    {
                        instanceArray[i].vol += instanceArray[i].fadeSpeed;
                        instanceArray[i].soundInstance.Volume = instanceArray[i].vol / 1000f;
                    }
                    else instanceArray[i].fadeIn = false;
                }

                if (instanceArray[i].fadeOut)
                {
                    if (instanceArray[i].vol > 0)
                    {
                        instanceArray[i].vol -= instanceArray[i].fadeSpeed;
                        if (instanceArray[i].vol < 0) instanceArray[i].vol = 0;
                        instanceArray[i].soundInstance.Volume = instanceArray[i].vol / 1000f;
                    }
                    else if(instanceArray[i].stopAfterFade == true)
                    {
                        instanceArray[i].fadeOut = false;
                        instanceArray[i].soundInstance.Stop();
                    }
                }
            }        
        }
    }
}
