﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Game
{
    public class Player
    {
        //Shoot variables
        int shootTime = 0;
        bool shoot = false;
        int shootSpeed = 8;

        //Draw variables
        int frameX = 0;
        int frameY = 0;
        int sizeX = 16;
        int sizeY = 16;
        Vector2 drawPos = new Vector2(0,0);
        double rotationAngle;
        Vector2 rotationPoint = new Vector2(8,8);

        //Shadow variables
        Vector2 shadowPos = new Vector2(0,0);
        Vector2 shadowCenter = new Vector2(4,4);

        //State variables
        public bool active = true;
        public bool dead = false;

        Point currentFrame; //BORRAR
        Texture2D playerSprite;

        SpriteEffects effect = SpriteEffects.None;

        //Move variables
        public Vector2 pos;
        int direction = 2; //0..7
        int closestMoveDir; //-1..7
        int changeDir;
        int prevChangeDir;
        int changeDirCont = 0;
        int changeDirTime = 3;
        int directionX = 0;
        int directionY = 0;
        int speedX = 2;
        int speedY = 2;
        int prevDirectionX = 0;
        int prevDirectionY = 0;

        //Teleport
        Vector2 teleportOrigin = new Vector2(0, 0);
        Vector2 teleportDir = new Vector2(0, 0);
        bool teleport = false;
        int teleportCont = 0;
        int teleportDist = 40;

        //Absorb
        public int absorbColor = -1;
        public bool absorb = false;
        public int absorbCont = 0;
        int absorbMaxTime = 80;
        public bool absorbed = false;
        int absorbTransparency = 0;
        public bool absorbOverheat = false;
        public bool stopAbsorb = false;

        //OverHeat
        bool overheat = false;
        int overheatCont = 0;
        int overheatTime = 160;

        public Player(Texture2D playerSprite)
        {
            this.currentFrame = new Point(0,0);
            this.pos = new Vector2(120, 144);
            this.playerSprite = playerSprite;
        }

        public void animationManager()
        {
            currentFrame.Y = 0;
            currentFrame.X = 0;
        }
        public void died()
        {
            dead = true;
        }
        public void resetState()
        {
            dead = false;
            effect = SpriteEffects.None;
            pos.X = 120;
            pos.Y = 144;
            directionX = 0;
            directionY = 0;
            speedX = 2;
            speedY = 2;
            prevDirectionX = 0;
            prevDirectionY = 0;
            absorbColor = -1;
            absorb = false;
            absorbCont = 0;
            currentFrame.Y = 0;
            absorbed = false;
            absorbTransparency = 0;
            overheat = false;
            overheatCont = 0;
        }

        public State update(GameTime gameTime, KeySet keySet, Background map, List<Enemy> enemyList, AudioManager audioManager,  BulletManager bulletManager)
        {
            //dead = false;
            if (dead)
            {
                audioManager.playSound(8, 600, 0);
                audioManager.stopInstance(0, -1);
                return State.mainMenu;                
            }
            else
            {
                //move

                pos.X += keySet.leftStickX*2.5f;
                pos.Y -= keySet.leftStickY*2.5f;

                //Calculate closest axis (0..7) to the player's movement
                if (keySet.leftStickX != 0 || keySet.leftStickY != 0)
                {
                    double H = Math.Sqrt((double)(Math.Pow(keySet.leftStickX, 2) + (double)Math.Pow(keySet.leftStickY, 2)));
                    double angle = Math.Asin(keySet.leftStickY / H);
                    if (keySet.leftStickX > 0)
                    {
                        if (angle < -1.1781) { closestMoveDir = 6; }
                        else if (angle >= -1.1781 && angle < -0.3927) { closestMoveDir = 7; }
                        else if (angle >= -0.3927 && angle < 0.3927) { closestMoveDir = 0; }
                        else if (angle >= 0.3927 && angle < 1.1781) { closestMoveDir = 1; }
                        else { closestMoveDir = 2; }

                    }
                    else
                    {
                        if (angle < -1.1781) closestMoveDir = 6;
                        else if (angle >= -1.1781 && angle < -0.3927) { closestMoveDir = 5; }
                        else if (angle >= -0.3927 && angle < 0.3927) { closestMoveDir = 4; }
                        else if (angle >= 0.3927 && angle < 1.1781) { closestMoveDir = 3; }
                        else { closestMoveDir = 2; }
                    }
                }
                else closestMoveDir = -1;

                //stop at screen boulders
                if (pos.X < 0) pos.X = 0;
                if (pos.X + 16 > 256) pos.X = 256 - 16;
                if (pos.Y < 0) pos.Y = 0;
                if (pos.Y + 16 > 240) pos.Y = 240 - 16;

                //Teleport                
                if (!teleport && keySet.L2Button && !keySet.prevL2Button)
                {
                    teleportOrigin.X = pos.X;
                    teleportOrigin.Y = pos.Y;
                    float moduleAux = (float)Math.Sqrt(keySet.leftStickX * keySet.leftStickX + keySet.leftStickY * keySet.leftStickY);

                    if (moduleAux != 0)
                    {
                        teleport = true;
                        teleportCont = 0;

                        teleportDir.X = keySet.leftStickX / moduleAux;
                        teleportDir.Y = keySet.leftStickY / moduleAux;

                        pos.X += teleportDir.X * teleportDist;
                        pos.Y -= teleportDir.Y * teleportDist;

                        bool offLimits = false;
                        if (pos.X < 0) { pos.X = 0; offLimits = true; }
                        if (pos.X + 16 > 256) { pos.X = 256 - 16; offLimits = true; }
                        if (pos.Y < 0) { pos.Y = 0; offLimits = true; }
                        if (pos.Y + 16 > 240) { pos.Y = 240 - 16; offLimits = true; }

                        if (offLimits)
                        {
                            float despX = pos.X - teleportOrigin.X;
                            float despY = pos.Y - teleportOrigin.Y;

                            moduleAux = (float)Math.Sqrt(despX * despX + despY * despY);
                            teleportDir.X = despX / moduleAux;
                            teleportDir.Y = -(despY / moduleAux);
                        }
                    }
                }
                else
                {
                    if (teleportCont == 4) teleport = false;
                    else teleportCont++;

                }

                //Shoot 
                prevChangeDir = changeDir;
                if (keySet.OButton && !keySet.TButton && !keySet.XButton)
                {
                    if (direction != 0)
                    {
                        changeDir = 0;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 0; rotationAngle = 1.5708f; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.OButton && keySet.TButton)
                {
                    if (direction != 1)
                    {
                        changeDir = 1;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 1; rotationAngle = 0.7854f; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.TButton && !keySet.OButton && !keySet.CButton)
                {
                    if (direction != 2)
                    {
                        changeDir = 2;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 2; rotationAngle = 0; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.TButton && keySet.CButton)
                {
                    if (direction != 3)
                    {
                        changeDir = 3;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 3; rotationAngle = -0.7854f; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.CButton && !keySet.TButton && !keySet.XButton)
                {
                    if (direction != 4)
                    {
                        changeDir = 4;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 4; rotationAngle = 4.7124; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.CButton && keySet.XButton)
                {
                    if (direction != 5)
                    {
                        changeDir = 5;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 5; rotationAngle = 3.927; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.XButton && !keySet.CButton && !keySet.OButton)
                {
                    if (direction != 6)
                    {
                        changeDir = 6;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 6; rotationAngle = 3.1416; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else if (keySet.XButton && keySet.OButton)
                {
                    if (direction != 7)
                    {
                        changeDir = 7;
                        if (changeDir == prevChangeDir)
                        {
                            if (changeDirCont == changeDirTime) { changeDirCont = 0; direction = 7; rotationAngle = 2.3562; shoot = true; }
                            else changeDirCont++;
                        }
                        else changeDirCont = 0;
                    }
                    else shoot = true; 
                }
                else shoot = false;

                if (closestMoveDir != -1)
                {
                    if ((direction + 2) % 8 == closestMoveDir) frameX = 1;
                    else if ((direction - 2 + 8) % 8 == closestMoveDir) frameX = 2;
                    else frameX = 0;
                }
                else frameX = 0;

                if (shoot)
                {
                    if (shootTime % 5 == 0) audioManager.playSound(3, 400, 0);

                    if (shootTime % 3 == 0)
                    {
                        switch (direction)
                        {
                            case 0:
                                bulletManager.addPlayerBullet(pos.X + 6.5f, pos.Y - 1.5f, (float)rotationAngle, shootSpeed, 0);
                                bulletManager.addPlayerBullet(pos.X + 6.5f, pos.Y + 3.5f, (float)rotationAngle, shootSpeed, 0);
                                break;
                            case 1:
                                bulletManager.addPlayerBullet(pos.X + 5, pos.Y - 1.5f, (float)rotationAngle, 0.67f * shootSpeed, -0.67f * shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 8.5f, pos.Y + 2, (float)rotationAngle, 0.67f * shootSpeed, -0.67f * shootSpeed);
                                break;
                            case 2:
                                bulletManager.addPlayerBullet(pos.X + 4, pos.Y + 1, (float)rotationAngle, 0, -shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 9, pos.Y + 1, (float)rotationAngle, 0, -shootSpeed);
                                break;
                            case 3:
                                bulletManager.addPlayerBullet(pos.X + 7.5f, pos.Y - 1.5f, (float)rotationAngle, -0.67f * shootSpeed, -0.67f * shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 4, pos.Y + 2, (float)rotationAngle, -0.67f * shootSpeed, -0.67f * shootSpeed);
                                break;
                            case 4:
                                bulletManager.addPlayerBullet(pos.X + 6.5f, pos.Y - 1.5f, (float)rotationAngle, -shootSpeed, 0);
                                bulletManager.addPlayerBullet(pos.X + 6.5f, pos.Y + 3.5f, (float)rotationAngle, -shootSpeed, 0);
                                break;
                            case 5:
                                bulletManager.addPlayerBullet(pos.X + 4f, pos.Y + 0, (float)rotationAngle, -0.67f * shootSpeed, 0.67f * shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 8f, pos.Y + 3f, (float)rotationAngle, -0.67f * shootSpeed, 0.67f * shootSpeed);
                                break;
                            case 6:
                                bulletManager.addPlayerBullet(pos.X + 4, pos.Y + 1, (float)rotationAngle, 0, shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 9, pos.Y + 1, (float)rotationAngle, 0, shootSpeed);
                                break;
                            case 7:
                                bulletManager.addPlayerBullet(pos.X + 5, pos.Y + 3f, (float)rotationAngle, 0.67f * shootSpeed, 0.67f * shootSpeed);
                                bulletManager.addPlayerBullet(pos.X + 8.5f, pos.Y + 0, (float)rotationAngle, 0.67f * shootSpeed, 0.67f * shootSpeed);
                                break;
                            default:
                                break;

                        }
                    }
                    shootTime++;
                }
                else shootTime = 0;

                //Shields

                if (absorbed) { absorbed = false; absorbCont = 0;}

                if (overheat)
                {
                    if (overheatCont == overheatTime)
                    {
                        overheat = false;
                        overheatCont = 0;
                        stopAbsorb = false;
                        absorb = false;
                    }
                    else overheatCont++;

                    if (keySet.R1Button || keySet.L1Button) absorbOverheat = true;
                    else absorbOverheat = false;
                }

                if (!stopAbsorb)
                {
                    if (keySet.L1Button)
                    {
                        if (!overheat && !absorbOverheat) absorb = true;
                        absorbColor = 0;
                    }
                    else if (keySet.R1Button)
                    {
                        if (!overheat && !absorbOverheat) absorb = true;
                        absorbColor = 1;
                    }
                    else
                    {
                        if (absorb) { absorb = false; stopAbsorb = true; }
                        absorbOverheat = false;
                    }
                }

                if (absorb)
                {
                    if (absorbTransparency < 100) absorbTransparency += 10;

                    if (absorbCont == absorbMaxTime - 24) audioManager.playSound(9, 900, 0);

                    if (absorbCont == absorbMaxTime)
                    {
                        audioManager.playSound(10, 900, 0);
                        overheat = true;
                        absorb = false;
                        absorbCont = 0;
                        absorbTransparency = 0; 
                    }
                    else absorbCont++;
                }
                else if(stopAbsorb)
                {
                    if (absorbTransparency > 0) absorbTransparency -= 10;
                    else { stopAbsorb = false; absorbCont = 0; }
                    
                }

                              
                //Shoot
                if (false)
                {
                    if (keySet.XButton)
                    {
                        if (shootTime % 5 == 0) audioManager.playSound(7, 400, 0);

                        if (shootTime % 3 == 0)
                        {

                            //regular double shoot
                            //bulletManager.addPlayerBullet((int)pos.X + 4 + directionX * speedX, (int)pos.Y - 8, 0f, 0, -8f);
                            //bulletManager.addPlayerBullet((int)pos.X + 9 + directionX * speedX, (int)pos.Y - 8, 0f, 0, -8f);

                            /*
                            //spiral double shoot
                            if (shootTime % 18 == 0)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 3, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 10, (int)pos.Y - 8, -8f);
                            }
                            else if (shootTime % 18 == 3)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 4, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 9, (int)pos.Y - 8, -8f);
                            }
                            else if (shootTime % 18 == 6)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 5, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 8, (int)pos.Y - 8, -8f);
                            }
                            else if (shootTime % 18 == 9)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 6, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 7, (int)pos.Y - 8, -8f);
                            }
                            else if (shootTime % 18 == 12)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 5, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 8, (int)pos.Y - 8, -8f);
                            }
                            else if (shootTime % 18 == 15)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 4, (int)pos.Y - 8, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 9, (int)pos.Y - 8, -8f);
                            }*/


                            //regular al moverse en horizontal y spiral al estar quieto:
                            //regular double shoot
                            if (directionX != 0)
                            {
                                bulletManager.addPlayerBullet((int)pos.X + 4 + directionX * speedX, (int)pos.Y - 8, 0f, 0, -8f);
                                bulletManager.addPlayerBullet((int)pos.X + 9 + directionX * speedX, (int)pos.Y - 8, 0f, 0, -8f);
                            }

                            //spiral double shoot
                            else
                            {
                                if (shootTime % 18 == 0)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 3, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 10, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                                else if (shootTime % 18 == 3)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 4, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 9, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                                else if (shootTime % 18 == 6)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 5, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 8, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                                else if (shootTime % 18 == 9)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 6, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 7, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                                else if (shootTime % 18 == 12)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 5, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 8, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                                else if (shootTime % 18 == 15)
                                {
                                    bulletManager.addPlayerBullet((int)pos.X + 4, (int)pos.Y - 8, 0f, 0, -8f);
                                    bulletManager.addPlayerBullet((int)pos.X + 9, (int)pos.Y - 8, 0f, 0, -8f);
                                }
                            }     

                        }
                        else shootTime++;
                    }
                }
                //else shootTime = 0;
                 
                //animationManager();

                shadowPos.X = 48 + pos.X * 0.625f;
                shadowPos.Y = 48 + pos.Y * 0.6f;
            }
            return State.game;
        }

        public void draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            drawPos.X = pos.X + 8;
            drawPos.Y = pos.Y + 8;

            rotationPoint.X = 8;
            rotationPoint.Y = 8;
            if (overheat)
            {
                spriteBatch.Draw(
                    playerSprite,
                    drawPos,
                    new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                    Color.Red, (float)rotationAngle, rotationPoint,
                    1f, effect, (float)0.5);
            }
            else
            {
                spriteBatch.Draw(
                    playerSprite,
                    drawPos,
                    new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                    Color.White, (float)rotationAngle, rotationPoint,
                    1f, effect, (float)0.5);
            }

            if (teleport)
            {
                drawPos.X = pos.X - teleportDir.X * 6 + 8;
                drawPos.Y = pos.Y + teleportDir.Y * 6 + 8;

                if (overheat)
                {
                    spriteBatch.Draw(
                       playerSprite,
                       drawPos,
                       new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                       Color.Red * 0.5f, (float)rotationAngle, rotationPoint,
                       1f, effect, (float)0.5);
                }
                else
                {
                    spriteBatch.Draw(
                       playerSprite,
                       drawPos,
                       new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                       Color.White * 0.5f, (float)rotationAngle, rotationPoint,
                       1f, effect, (float)0.5);
                }

                drawPos.X = pos.X - teleportDir.X * 12 + 8;
                drawPos.Y = pos.Y + teleportDir.Y * 12 + 8;

                if (overheat)
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                        Color.Red * 0.5f, (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.5);
                }
                else
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                        Color.White * 0.5f, (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.5);
                }

                drawPos.X = pos.X - teleportDir.X * 18 + 8;
                drawPos.Y = pos.Y + teleportDir.Y * 18 + 8;

                if (overheat)
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                        Color.Red * 0.25f, (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.5);
                }
                else
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * sizeX, 0, sizeX, sizeY),
                        Color.White * 0.25f, (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.5);
                }
            }
            
            rotationPoint.X = 9;
            rotationPoint.Y = 9;

            if (absorbCont >= 0 && absorbCont < absorbMaxTime - 24) //overheat advise
            {
                if (absorbColor == 0)
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * 18, 16, 18, 18),
                        Color.White * (absorbTransparency / 100f), (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.49);
                }
                else
                {
                    spriteBatch.Draw(
                        playerSprite,
                        drawPos,
                        new Rectangle(frameX * 18, 34, 18, 18),
                        Color.White * (absorbTransparency / 100f), (float)rotationAngle, rotationPoint,
                        1f, effect, (float)0.49);
                }
            }
            else
            {
                spriteBatch.Draw(
                    playerSprite,
                    drawPos,
                    new Rectangle(frameX * 18, 52, 18, 18),
                    Color.White * (absorbTransparency / 100f), (float)rotationAngle, rotationPoint,
                    1f, effect, (float)0.49);
            }

            //Shadow
            spriteBatch.Draw(
                playerSprite,
                shadowPos,
                new Rectangle(48 + frameX * 8, 0, 8, 8),
                Color.White * 0.05f, (float)rotationAngle, shadowCenter,
                1f, effect, (float)0.85f);
            spriteBatch.Draw(
                playerSprite,
                shadowPos,
                new Rectangle(48 + frameX * 8, 0, 8, 8),
                Color.White * 0.05f, (float)rotationAngle, shadowCenter,
                1f, effect, (float)0.75f);

        }
    }
}