﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Game
{
    public class BGElement
    {
        public int typeID;
        //0 nube pequeña
        //1 nube grande
        public Vector2 pos;
        public int speedX;
        public float speedY;
        public bool remove = false;

        public BGElement(int typeID, int x, int y, int speedX, float speedY){
            this.typeID = typeID;
            pos.X = x;
            pos.Y = y;
            this.speedX = speedX;
            this.speedY = speedY;
        }

        public void update()
        {
            pos.Y += speedY;
            if (pos.Y > 240) remove = true;
        }
    }

    public struct tileBackground
    {
        public int width, height;
        public int[][] map;
        public int scroll;
        public int scrollSpeed;
    }

    public class Background
    {
        public bool drawClouds = true;
        public bool drawBackground = true;
        public bool drawFlatColor = false;

        bool init = true;

        //background cloud stuff
        List<BGElement> BGElementList = new List<BGElement>();
        BGElement BGElementAux = new BGElement(0, 0, 0, 0, 0);

        Random rand = new Random(1);

        int cloudSTime = 16;
        int cloudScont = 0;
        int cloudBTime = 32;
        int cloudBcont = 0;

        //Tile background
        tileBackground tileBackground;
        public string line;
        string[] valueString;

        public Texture2D backgroundTileSheet;
        Vector2 drawPos;
        int stage = 0;
        float fadeLevelBlack;

        public Background(Texture2D backgroundTileSheet)
        {
            this.backgroundTileSheet = backgroundTileSheet;
        }

        public void loadTileMap(int id)
        {
            String fileMap = "map" + id + ".txt";
            StreamReader sr = new StreamReader(fileMap);
            line = sr.ReadLine();
            valueString = line.Split(new char[] { ' ', '\n' }, StringSplitOptions.RemoveEmptyEntries);
            tileBackground.width = 16;
            tileBackground.height = int.Parse(valueString[1]);
            tileBackground.scroll = 0;
            tileBackground.scrollSpeed = 1;

            tileBackground.map = new int[16][];
            for (int i = 0; i < tileBackground.width; i++)
            {
                tileBackground.map[i] = new int[tileBackground.height];
            }

            for (int i = 0; i < tileBackground.height; i++)
            {
                line = sr.ReadLine();
                valueString = line.Split(new char[] { ' ', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                for (int j = 0; j < tileBackground.width; j++)
                {
                    tileBackground.map[j][i] = int.Parse(valueString[j]);
                }
            }
        }

        public void update(Camera2d cam, Player chica, StageManager stageManager, AudioManager audioManager)
        {
            if (init)
            {
                init = false;
                loadTileMap(1);
            }

            //generate small clouds

            if (cloudScont == cloudSTime - 1)
            {
                cloudScont = 0;
                if (rand.Next(0, /*4*/6) != 0)
                {
                    int numClouds = rand.Next(0, 1/*1*/);
                    for (int i = 0; i <= numClouds; i++)
                    {
                        BGElementAux.typeID = 0;
                        BGElementAux.pos.X = rand.Next(2, 19) * 16 - 8;
                        BGElementAux.pos.Y = -32;
                        if (BGElementAux.pos.X % 32 == 8) BGElementAux.speedY = 2;
                        else BGElementAux.speedY = 2.5f;
                        BGElementList.Add(new BGElement(BGElementAux.typeID, (int)BGElementAux.pos.X, (int)BGElementAux.pos.Y, 0, BGElementAux.speedY));
                    }
                }
            }
            else cloudScont++;

            //generate big clouds

            if (cloudBcont == cloudBTime - 1)
            {
                cloudBcont = 0;
                if (rand.Next(0, 2) == 0)
                {
                    BGElementAux.typeID = 1;
                    BGElementAux.pos.X = rand.Next(1, 10) * 32 - 16;
                    BGElementAux.pos.Y = -64;
                    BGElementAux.speedY = 3;
                    BGElementList.Add(new BGElement(BGElementAux.typeID, (int)BGElementAux.pos.X, (int)BGElementAux.pos.Y, 0, BGElementAux.speedY));
                }
            }
            else cloudBcont++;

            //Update background elements

            for (int p = 0; p < BGElementList.Count(); p++)
            {
                BGElementList.ElementAt(p).update();
            }

            for (int p = 0; p < BGElementList.Count();)
            {
                if (BGElementList.ElementAt(p).remove == true) BGElementList.RemoveAt(p);
                else p++;
            }

            //Update scroll
            tileBackground.scroll += tileBackground.scrollSpeed;
            
        }

        public void draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            drawPos.X = 0;
            drawPos.Y = 0;

            //Draw plain color background
            if (drawFlatColor)
            {
                spriteBatch.Draw(
                    backgroundTileSheet,
                    drawPos,
                    new Rectangle(0, 0, 16, 16),
                    Color.White, 0, Vector2.Zero,
                    24f, SpriteEffects.None, 1);
            }
            

            //Draw tile background

            if (drawBackground)
            {
                int tileID;
                for (int i = 0; i < 16; i++)
                {
                    drawPos.X = i * 16;
                    for (int j = -1; j < 15; j++)
                    {
                        drawPos.Y = j * 16 + tileBackground.scroll % 16;
                        tileID = tileBackground.map[i][(tileBackground.height + j - (tileBackground.scroll / 16 % tileBackground.height)) % tileBackground.height] - 1;

                        spriteBatch.Draw(
                            backgroundTileSheet,
                            drawPos,
                            new Rectangle((tileID % 8) * 16 + 80, (tileID / 8) * 16, 16, 16),
                            Color.White, 0, Vector2.Zero,
                            1f, SpriteEffects.None, 0.95f);
                    }
                }
            }

            //Draw background elements
            if (drawClouds)
            {
                for (int p = 0; p < BGElementList.Count(); p++)
                {
                    switch (BGElementList.ElementAt(p).typeID)
                    {
                        case 0: //small cloud
                            {

                                spriteBatch.Draw(
                                    backgroundTileSheet,
                                    BGElementList.ElementAt(p).pos,
                                    new Rectangle(0 + 1, 16 + 1, 16, 32),
                                    Color.White, 0, Vector2.Zero,
                                    1f, SpriteEffects.None, (float)0.8);
                                break;
                            }
                        case 1: //big cloud
                            {
                                //Con transparencia
                                spriteBatch.Draw(
                                    backgroundTileSheet,
                                    BGElementList.ElementAt(p).pos,
                                    new Rectangle(18 + 1, 0 + 1, 32, 64),
                                    Color.White, 0, Vector2.Zero,
                                    1f, SpriteEffects.None, (float)0.7);
                                spriteBatch.Draw(
                                    backgroundTileSheet,
                                    BGElementList.ElementAt(p).pos,
                                    new Rectangle(18 + 1, 0 + 1, 32, 64),
                                    Color.White * 0.5f, 0, Vector2.Zero,
                                    1f, SpriteEffects.None, (float)0.3);

                                break;
                            }
                        default:
                            {
                                break;
                            }

                    }
                }
            }
  
        }

    }
       
}

