using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Game
{
    public enum State { intro, mainMenu, game, exit};

    public class Game : Microsoft.Xna.Framework.Game
    {

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        RenderTarget2D mainTarget;  
        
        State currentState = State.intro;
        State newState = State.intro;

        //intro
        IntroScreenManager introScreenManager;

        //main menu
        public MainMenu menu;

        //Audio
        AudioManager audioManager;

        //Input
        InputManager inputManager = new InputManager();
        KeySet keySet;

        //game
        public StageManager stageManager;
        public ScoreManager scoreManager;
        public EnemyManager enemyManager;
        public Player player;
        BulletManager bulletManager;
        public Camera2d cam = new Camera2d();
        public Camera2d interfaceCam = new Camera2d();
        public Texture2D fontSheet;
        public Background backGround;

        public Game()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            
            //cambiar frame-rate;
            //TargetElapsedTime = new TimeSpan(0, 0, 0, 0, 50);
            
            //cambiar resolucin
            graphics.PreferredBackBufferWidth = 256 * 2;// 1360;// 256 * 3;// 1360;
            graphics.PreferredBackBufferHeight = 240 * 2;// 768;//240 * 3;// 768;

            //Vsyinc
            //graphics.SynchronizeWithVerticalRetrace = false;

            //mouse
            //IsMouseVisible = true;
        }
        
        protected override void Initialize()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Window.AllowUserResizing = false;
            GraphicsDevice.SetRenderTarget(mainTarget);            

            //pantalla completa            
            //graphics.IsFullScreen = true;

            //Change viewport
            /*
            Viewport leftViewport;
            leftViewport = GraphicsDevice.Viewport;
            leftViewport.X =  (1360 - 256 * 3) / 2;
            leftViewport.Y =  (768 - 240 * 3) / 2;
            leftViewport.Width = 256 * 3;
            leftViewport.Height = 240 * 3;
            GraphicsDevice.Viewport = leftViewport;
            */

            //manager = new Manager(this);

            IsFixedTimeStep = false;

            introScreenManager = new IntroScreenManager(Content.Load<Texture2D>(@"Images/introScreen"), Content.Load<Texture2D>(@"Images/blackBackGround"));
            scoreManager = new ScoreManager();
            scoreManager.font = Content.Load<SpriteFont>("editor");
            backGround = new Background(Content.Load<Texture2D>(@"Images/backgroundSheet"));
            enemyManager = new EnemyManager(Content.Load<Texture2D>(@"Images/enemySprites"));
            stageManager = new StageManager(Content.Load<Texture2D>(@"Images/stageSign"));
            stageManager.font = Content.Load<SpriteFont>("editor");
            player = new Player(Content.Load<Texture2D>(@"Images/chica"));
            bulletManager = new BulletManager(Content.Load<Texture2D>(@"Images/playerBulletSheet"), Content.Load<Texture2D>(@"Images/enemyBulletSheet"));
            fontSheet = Content.Load<Texture2D>(@"Images/fontSheet");
            menu = new MainMenu(Content.Load<Texture2D>(@"Images/letters"), Content.Load<Texture2D>(@"Images/lettersWhite"), Content.Load<Texture2D>(@"Images/gameLogo"), Content.Load<Texture2D>(@"Images/menuBackground"));
            audioManager = new AudioManager(Content);

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // TODO: use this.Content to load your game content here

            base.LoadContent();
        }

        public void manageStateChange(MainMenu mainMenu, AudioManager audioManager, Background map)
        {
            switch (currentState)
            {
                case State.intro:
                    {
                        currentState = newState;
                        break;
                    }
                case State.mainMenu:
                    {
                        currentState = newState;
                        if (newState == State.game) { }
                        else if (newState == State.exit) this.Exit();
                        break;
                    }
            }
        }

        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed) this.Exit();

            //manager.Update(gameTime);
            //En vez de la parida de crear un game que sirva como contentLoader, pasar directamente el del game:
            //ContentManager Content;

            inputManager.update();
            keySet = inputManager.getKeySet();

            if (currentState == State.intro)
            {
                newState = introScreenManager.update(gameTime, audioManager);
                if (newState != State.intro) { currentState = newState; }
            }

            if (currentState == State.mainMenu)
            {
                newState = menu.update(keySet, audioManager, scoreManager, enemyManager, backGround);
                if (newState != State.mainMenu) manageStateChange(menu, audioManager, backGround);
            }

            if (currentState == State.game)
            {
                newState = player.update(gameTime, keySet, backGround, enemyManager.enemyList, audioManager, bulletManager);
                if (newState == State.mainMenu)
                {
                    currentState = newState;
                    if (scoreManager.checkNewHighScore())
                    {
                        scoreManager.insertHighScore();
                        menu.currentScreen = 8;
                    }
                    else
                    {
                        menu.currentScreen = 7;
                    }
                    player.resetState();
                    enemyManager.resetState();
                    stageManager.resetState();
                    bulletManager.resetState();
                }
                else
                {
                    bulletManager.update(gameTime, player, enemyManager.enemyList, audioManager, scoreManager, stageManager.time);
                    enemyManager.update(gameTime, player, cam, bulletManager.playerBulletList, bulletManager.enemyBulletList, audioManager, scoreManager, stageManager.time);
                    backGround.update(cam, player, stageManager, audioManager);
                    stageManager.update(gameTime, player, cam, bulletManager.enemyBulletList, enemyManager, audioManager, scoreManager);
                }
            }

            cam.update(player);

            audioManager.update();
            
            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {      
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin(SpriteSortMode.BackToFront,
                        BlendState.AlphaBlend,
                        SamplerState.PointClamp,
                        DepthStencilState.Default,
                        RasterizerState.CullNone,
                        null,
                        cam.get_transformation(graphics));

            //Draw intro
            if (currentState == State.intro) introScreenManager.draw(gameTime, spriteBatch);

            //Draw game
            if (currentState == State.game)
            {
                backGround.draw(gameTime, spriteBatch);
                player.draw(gameTime, spriteBatch);
                bulletManager.draw(gameTime, spriteBatch);
                enemyManager.draw(gameTime, spriteBatch);
                scoreManager.draw(gameTime, spriteBatch);
                stageManager.draw(gameTime, spriteBatch);
            }

            //Draw menu
            if (currentState == State.mainMenu) menu.draw(gameTime, spriteBatch, scoreManager);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
