﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Game
{
    public struct KeySet
    {
        public float
        leftStickX,
        leftStickY,
        rightStickX,
        rightStickY;

        public int
            stickDirection,
            prevStickDirection;

        public bool 
        rightButton,
        leftButton,
        upButton,
        downButton,
        XButton,
        OButton,
        CButton,
        TButton,
        L1Button,
        R1Button,
        L2Button;

        public bool
        releasedRightButton,
        releasedLeftButton,
        releasedUpButton,
        releasedDownButton,
        releasedXButton,
        releasedOButton,
        releasedQButton,
        releasedTButton,
        releasedL1Button,
        releasedR1Button,
        releasedL2Button;

        public bool 
        prevRightButton,
        prevLeftButton,
        prevUpButton,
        prevDownButton,
        prevXButton,
        prevOButton,
        prevQButton,
        prevTButton,
        prevL1Button,
        prevR1Button,
        prevL2Button;
    }

    public class InputManager
    {
        KeyboardState keyState = Keyboard.GetState();
        KeySet keySet;
        bool gamePadIsActive = false;

        public InputManager(){

            keySet.rightButton = false;
            keySet.leftButton = false;
            keySet.upButton = false;
            keySet.downButton = false;
            keySet.XButton = false;
            keySet.OButton = false;
            keySet.CButton = false;
            keySet.TButton = false;
            keySet.L1Button = false;
            keySet.R1Button = false;

            keySet.releasedRightButton = false;
            keySet.releasedLeftButton = false;
            keySet.releasedUpButton = false;
            keySet.releasedDownButton = false;
            keySet.releasedXButton = false;
            keySet.releasedOButton = false;
            keySet.releasedQButton = false;
            keySet.releasedTButton = false;
            keySet.releasedL1Button = false;
            keySet.releasedR1Button = false;

            keySet.prevRightButton = false;
            keySet.prevLeftButton = false;
            keySet.prevUpButton = false;
            keySet.prevDownButton = false;
            keySet.prevXButton = false;
            keySet.prevOButton = false;
            keySet.prevQButton = false;
            keySet.prevTButton = false;
            keySet.prevL1Button = false;
            keySet.prevR1Button = false;

        }

        public void update()
        {
            //Previous keys:

            keySet.prevUpButton = keySet.upButton;
            keySet.prevRightButton = keySet.rightButton;
            keySet.prevDownButton = keySet.downButton;
            keySet.prevLeftButton = keySet.leftButton;
            keySet.prevXButton = keySet.XButton;
            keySet.prevOButton = keySet.OButton;
            keySet.prevQButton = keySet.CButton;
            keySet.prevTButton = keySet.TButton;
            keySet.prevL1Button = keySet.L1Button;
            keySet.prevR1Button = keySet.R1Button;
            keySet.prevL2Button = keySet.L2Button;
            keySet.prevStickDirection = keySet.stickDirection;


            //Pressed keys:

            keySet.upButton = false;
            keySet.rightButton = false;
            keySet.downButton = false;
            keySet.leftButton = false;
            keySet.XButton = false;
            keySet.OButton = false;
            keySet.CButton = false;
            keySet.TButton = false;
            keySet.L1Button = false;
            keySet.R1Button = false;
            keySet.L2Button = false;
            keySet.stickDirection = -1;

            gamePadIsActive = GamePad.GetState(PlayerIndex.One).IsConnected;

            if (gamePadIsActive) manageGamePadInput();
            else manageKeyboardInput();

            //Released buttons:

            keySet.releasedUpButton = false;
            keySet.releasedRightButton = false;
            keySet.releasedDownButton = false;
            keySet.releasedLeftButton = false;
            keySet.releasedXButton = false;
            keySet.releasedOButton = false;
            keySet.releasedQButton = false;
            keySet.releasedTButton = false;
            keySet.releasedL1Button = false;
            keySet.releasedR1Button = false;
            keySet.releasedL2Button = false;

            if (keySet.prevUpButton == true && keySet.upButton == false) keySet.releasedUpButton = true;
            if (keySet.prevRightButton == true && keySet.rightButton == false) keySet.releasedRightButton = true;
            if (keySet.prevDownButton == true && keySet.downButton == false) keySet.releasedDownButton = true;
            if (keySet.prevLeftButton == true && keySet.leftButton == false) keySet.releasedLeftButton = true;
            if (keySet.prevXButton == true && keySet.XButton == false) keySet.releasedXButton = true;
            if (keySet.prevOButton == true && keySet.OButton == false) keySet.releasedOButton = true;
            if (keySet.prevQButton == true && keySet.CButton == false) keySet.releasedQButton = true;
            if (keySet.prevTButton == true && keySet.TButton == false) keySet.releasedTButton = true;
            if (keySet.prevL1Button == true && keySet.L1Button == false) keySet.releasedL1Button = true;
            if (keySet.prevR1Button == true && keySet.R1Button == false) keySet.releasedR1Button = true;
            if (keySet.prevL2Button == true && keySet.L2Button == false) keySet.releasedL2Button = true;          
        }

        void manageKeyboardInput(){

            keyState = Keyboard.GetState();

            if (keyState.IsKeyDown(Keys.Up)) keySet.upButton = true;
            else if (keyState.IsKeyDown(Keys.Down)) keySet.downButton = true;
            if (keyState.IsKeyDown(Keys.Right)) keySet.rightButton = true;
            else if (keyState.IsKeyDown(Keys.Left)) keySet.leftButton = true;

            if (keyState.IsKeyDown(Keys.X)) { keySet.XButton = true; }

            if (keyState.IsKeyDown(Keys.S)) { keySet.OButton = true; }

            if (keyState.IsKeyDown(Keys.D)) { keySet.CButton = true; }

            if (keyState.IsKeyDown(Keys.C)) { keySet.TButton = true; }

            if (keyState.IsKeyDown(Keys.N)) { keySet.L1Button = true; }

        }

        void manageGamePadInput()
        {
            keySet.leftStickX = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.X;
            keySet.leftStickY = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y;
            keySet.rightStickX = GamePad.GetState(PlayerIndex.One).ThumbSticks.Right.X;
            keySet.rightStickY = GamePad.GetState(PlayerIndex.One).ThumbSticks.Right.Y;

            if (keySet.leftStickX > 0.7f) keySet.stickDirection = 1;
            else if (keySet.leftStickX < -0.7f) keySet.stickDirection = 3;

            if (keySet.leftStickY < -0.7f) keySet.stickDirection = 2;
            else if (keySet.leftStickY > 0.7f) keySet.stickDirection = 0;

            if (GamePad.GetState(PlayerIndex.One).DPad.Up == ButtonState.Pressed) keySet.upButton = true;
            else if (GamePad.GetState(PlayerIndex.One).DPad.Down == ButtonState.Pressed) keySet.downButton = true;
            if (GamePad.GetState(PlayerIndex.One).DPad.Right == ButtonState.Pressed) keySet.rightButton = true;
            else if (GamePad.GetState(PlayerIndex.One).DPad.Left == ButtonState.Pressed) keySet.leftButton = true;

            if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed) keySet.XButton = true;
            if (GamePad.GetState(PlayerIndex.One).Buttons.B == ButtonState.Pressed) keySet.OButton = true;
            if (GamePad.GetState(PlayerIndex.One).Buttons.X == ButtonState.Pressed) keySet.CButton = true;
            if (GamePad.GetState(PlayerIndex.One).Buttons.Y == ButtonState.Pressed) keySet.TButton = true;
            if (GamePad.GetState(PlayerIndex.One).Buttons.LeftShoulder == ButtonState.Pressed) keySet.L1Button = true;
            if (GamePad.GetState(PlayerIndex.One).Buttons.RightShoulder == ButtonState.Pressed) keySet.R1Button = true;
            if (GamePad.GetState(PlayerIndex.One).Triggers.Left >= 0.3f) keySet.L2Button = true;

        }

        public KeySet getKeySet()
        {
            return keySet;
        }

    }
}
