﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Game
{

    public struct MainMenuOption
    {
        public Vector2 pos;
        public int[] kanjiString;
        public int[] romajiString;
        public int optionID;
        
        public int type; //0 regular option, 1 atributo (te permite seleccionar entre distintos valores).
        public List<int[]> values;
        public int numValues;
        public int currentValue;
    }

    public struct MenuScreen
    {
        public int numOptions;
        public int numSelectableOptions;
        public MainMenuOption[] optionArray;
        public int screenID;
        public int prevScreenID;
        public int prevSelectedOption;
    }

    public enum SaveFileOption { newGame, continueGame, rename, move, copy, delete };

    public class MainMenu
    {
        public SaveFileOption saveFileOption;

        Texture2D kanjiImage;
        Texture2D kanjiImageBlack;
        Texture2D logoImage;
        Texture2D backgroundImage;

        Vector2 pos = new Vector2(0, 0);

        public MenuScreen[] screenArray = new MenuScreen[10];

        public int currentScreen = 0;
        int selectedOption = 0;
        
        int symbol = 0;
        int namePos = 0;

        public MainMenu(
            Texture2D kanjiImage,
            Texture2D kanjiImageBlack,
            Texture2D logoImage,
            Texture2D backgroundImage)
        {
            this.kanjiImage = kanjiImage;
            this.kanjiImageBlack = kanjiImageBlack;
            this.logoImage = logoImage;
            this.backgroundImage = backgroundImage;
            setMenu();
        }

        public void setMenu()
        {
            //Press Start Screen
            screenArray[0].numOptions = 1;
            screenArray[0].numSelectableOptions = 1;
            screenArray[0].screenID = 0;
            screenArray[0].prevScreenID = -1;
            screenArray[0].optionArray = new MainMenuOption[screenArray[0].numOptions];

            screenArray[0].optionArray[0].pos.X = 64;
            screenArray[0].optionArray[0].pos.Y = 128;
            screenArray[0].optionArray[0].optionID = 0;
            screenArray[0].optionArray[0].kanjiString = new int[] { 255, 232, 191, 162, 255, 162, 165, 257, 169 };
            screenArray[0].optionArray[0].romajiString = new int[] { 45, 17, 4, 18, 18, 255, 18, 19, 0, 17, 19 }; //Press start

            //Main menu screen
            screenArray[1].numOptions = 4;
            screenArray[1].numSelectableOptions = 4;
            screenArray[1].screenID = 1;
            screenArray[1].prevScreenID = 0;
            screenArray[1].prevSelectedOption = 0;
            screenArray[1].optionArray = new MainMenuOption[screenArray[1].numOptions];

            screenArray[1].optionArray[0].pos.X = 40;
            screenArray[1].optionArray[0].pos.Y = 32;
            screenArray[1].optionArray[0].optionID = 1;
            screenArray[1].optionArray[0].kanjiString = new int[] { 85, 126, 93, 100 }; //hajimeru
            screenArray[1].optionArray[0].romajiString = new int[] { 31, 4, 6, 8, 13 }; //Begin

            screenArray[1].optionArray[1].pos.X = 40;
            screenArray[1].optionArray[1].pos.Y = 80;
            screenArray[1].optionArray[1].optionID = 2;
            screenArray[1].optionArray[1].kanjiString = new int[] { 175, 151, 162, 159, 150 }; //haisukoa
            screenArray[1].optionArray[1].romajiString = new int[] { 37, 8, 6, 7, 255, 18, 2, 14, 17, 4, 18}; //high scores

            screenArray[1].optionArray[2].pos.X = 40;
            screenArray[1].optionArray[2].pos.Y = 128;
            screenArray[1].optionArray[2].optionID = 3;
            screenArray[1].optionArray[2].kanjiString = new int[] { 177, 189, 257, 184, 257, 224}; //jiyuumo-do
            screenArray[1].optionArray[2].romajiString = new int[] { 35, 17, 4, 4, 255, 13, 14, 3, 4}; //Free mode

            screenArray[1].optionArray[3].pos.X = 40;
            screenArray[1].optionArray[3].pos.Y = 176;
            screenArray[1].optionArray[3].optionID = 4;
            screenArray[1].optionArray[3].kanjiString = new int[] { 154, 232, 161, 204, 195 }; //opushon
            screenArray[1].optionArray[3].romajiString = new int[] { 44, 15, 19, 8, 14, 13, 18 }; //Options

            //Options screen
            screenArray[2].numOptions = 4;
            screenArray[2].numSelectableOptions = 4;
            screenArray[2].screenID = 2;
            screenArray[2].prevScreenID = 1;
            screenArray[2].prevSelectedOption = 3;
            screenArray[2].optionArray = new MainMenuOption[screenArray[2].numOptions];

            screenArray[2].optionArray[0].pos.X = 40;
            screenArray[2].optionArray[0].pos.Y = 32;
            screenArray[2].optionArray[0].optionID = 5;
            screenArray[2].optionArray[0].kanjiString = new int[] { 213, 257, 182, 104, 133, 100 }; //geemu wo deru
            screenArray[2].optionArray[0].romajiString = new int[] { 34, 23, 8, 19, 255, 6, 0, 12, 4 }; //Exit game

            screenArray[2].optionArray[1].pos.X = 40;
            screenArray[2].optionArray[1].pos.Y = 80;
            screenArray[2].optionArray[1].optionID = 6;
            screenArray[2].optionArray[1].kanjiString = new int[] { 73, 61, 70, 67, 71, 112 }; //taitoru
            screenArray[2].optionArray[1].romajiString = new int[] { 32, 17, 04, 03, 08, 19, 18 }; //Credits

            screenArray[2].optionArray[2].pos.X = 40;
            screenArray[2].optionArray[2].pos.Y = 128;
            screenArray[2].optionArray[2].optionID = 7;
            screenArray[2].optionArray[2].kanjiString = new int[] { 123, 105, 124 }; //Gengo
            screenArray[2].optionArray[2].romajiString = new int[] { 41, 0, 13, 6, 20, 0, 6, 4 }; //Language
            screenArray[2].optionArray[2].type = 1;
            screenArray[2].optionArray[2].numValues = 2;
            screenArray[2].optionArray[2].currentValue = 0;
            screenArray[2].optionArray[2].values = new List<int[]>();
            screenArray[2].optionArray[2].values.Add(new int[] { 34, 13, 6}); //Eng
            screenArray[2].optionArray[2].values.Add(new int[] { 34, 18, 15}); //Esp

            screenArray[2].optionArray[3].pos.X = 40;
            screenArray[2].optionArray[3].pos.Y = 176;
            screenArray[2].optionArray[3].optionID = 8;
            screenArray[2].optionArray[3].kanjiString = new int[] { 68, 72 }; //kesu
            screenArray[2].optionArray[3].romajiString = new int[] { 33, 4, 11, 4, 19, 4 }; //delete

            //erase score table screen.
            screenArray[6].numOptions = 2;
            screenArray[6].numSelectableOptions = 2;
            screenArray[6].screenID = 6;
            screenArray[6].prevScreenID = 2;
            screenArray[6].prevSelectedOption = 3;
            screenArray[6].optionArray = new MainMenuOption[screenArray[6].numOptions];

            screenArray[6].optionArray[0].pos.X = 40;
            screenArray[6].optionArray[0].pos.Y = 80;
            screenArray[6].optionArray[0].optionID = 17;
            screenArray[6].optionArray[0].kanjiString = new int[] { 85, 61 }; //hai
            screenArray[6].optionArray[0].romajiString = new int[] { 54, 4, 18 }; //yes

            screenArray[6].optionArray[1].pos.X = 40;
            screenArray[6].optionArray[1].pos.Y = 128;
            screenArray[6].optionArray[1].optionID = 18;
            screenArray[6].optionArray[1].kanjiString = new int[] { 61, 61, 63 }; //iie
            screenArray[6].optionArray[1].romajiString = new int[] { 43, 14 }; //no

            //high scores
            screenArray[7].numOptions = 0;
            screenArray[7].numSelectableOptions = 0;
            screenArray[7].screenID = 7;
            screenArray[7].prevScreenID = 1;
            screenArray[7].prevSelectedOption = 1;
            screenArray[7].optionArray = new MainMenuOption[screenArray[7].numOptions];

            //input new score
            screenArray[8].numOptions = 0;
            screenArray[8].numSelectableOptions = 0;
            screenArray[8].screenID = 8;
            screenArray[8].prevScreenID = 1;
            screenArray[8].optionArray = new MainMenuOption[screenArray[8].numOptions];

            //Free mode
            screenArray[9].numOptions = 3;
            screenArray[9].numSelectableOptions = 3;
            screenArray[9].screenID = 9;
            screenArray[9].prevScreenID = 1;
            screenArray[9].prevSelectedOption = 2;
            screenArray[9].optionArray = new MainMenuOption[screenArray[9].numOptions];

            screenArray[9].optionArray[0].pos.X = 40;
            screenArray[9].optionArray[0].pos.Y = 32;
            screenArray[9].optionArray[0].optionID = -1;
            screenArray[9].optionArray[0].kanjiString = new int[] { 162, 168, 257, 216 }; //sute-ji
            screenArray[9].optionArray[0].romajiString = new int[] { 48, 19, 0, 6, 4}; //Stage
            screenArray[9].optionArray[0].type = 1;
            screenArray[9].optionArray[0].numValues = 5;
            screenArray[9].optionArray[0].currentValue = 0;
            screenArray[9].optionArray[0].values = new List<int[]>();
            screenArray[9].optionArray[0].values.Add(new int[] { 48, 241 });
            screenArray[9].optionArray[0].values.Add(new int[] { 48, 242 });
            screenArray[9].optionArray[0].values.Add(new int[] { 31, 241 });
            screenArray[9].optionArray[0].values.Add(new int[] { 31, 242 });
            screenArray[9].optionArray[0].values.Add(new int[] { 31, 243 });

            screenArray[9].optionArray[1].pos.X = 40;
            screenArray[9].optionArray[1].pos.Y = 80;
            screenArray[9].optionArray[1].optionID = -1;
            screenArray[9].optionArray[1].kanjiString = new int[] { 85, 61, 68, 61 }; //haikei
            screenArray[9].optionArray[1].romajiString = new int[] { 31, 0, 2, 10, 6, 17, 14, 20, 13, 3 }; //Background
            screenArray[9].optionArray[1].type = 1;
            screenArray[9].optionArray[1].numValues = 3;
            screenArray[9].optionArray[1].currentValue = 0;
            screenArray[9].optionArray[1].values = new List<int[]>();
            screenArray[9].optionArray[1].values.Add(new int[] { 35, 14, 17 });
            screenArray[9].optionArray[1].values.Add(new int[] { 48, 10, 24});
            screenArray[9].optionArray[1].values.Add(new int[] { 43, 14 });

            screenArray[9].optionArray[2].pos.X = 40;
            screenArray[9].optionArray[2].pos.Y = 176;
            screenArray[9].optionArray[2].optionID = 10;
            screenArray[9].optionArray[2].kanjiString = new int[] { 85, 126, 93, 100 }; //hajimeru
            screenArray[9].optionArray[2].romajiString = new int[] { 31, 4, 6, 8, 13 }; //Begin

        }

        public State update(KeySet keyset, AudioManager audioManager, ScoreManager scoreManager, EnemyManager enemyManager, Background background)
        {
            if(currentScreen == 8) //input new score
            {
                if (keyset.rightButton && !keyset.prevRightButton || keyset.stickDirection == 1 && keyset.prevStickDirection == -1)
                {
                    audioManager.playSound(1, 1000, 0);
                    symbol++;
                    if(symbol == 26) symbol = 0;
                    scoreManager.highScores[scoreManager.newScorePos].name[namePos] = symbol;

                }
                else if (keyset.leftButton && !keyset.prevLeftButton || keyset.stickDirection == 3 && keyset.prevStickDirection == -1)
                {
                    audioManager.playSound(1, 1000, 0);
                    symbol--;
                    if(symbol == -1) symbol = 25;
                    scoreManager.highScores[scoreManager.newScorePos].name[namePos] = symbol;
                }

                if (keyset.L1Button && !keyset.prevL1Button)
                {
                    namePos++;
                    if (namePos == 3)
                    {
                        scoreManager.saveScores();
                        currentScreen = 7;
                        namePos = 0;
                        symbol = 0;
                    }
                    else
                    {
                        symbol = 0;
                        scoreManager.highScores[scoreManager.newScorePos].name[namePos] = symbol;
                    }
                }
                else if (keyset.XButton && !keyset.prevXButton)
                {
                    if (namePos > 0)
                    {
                        scoreManager.highScores[scoreManager.newScorePos].name[namePos] = 255;
                        namePos--;
                        symbol = scoreManager.highScores[scoreManager.newScorePos].name[namePos];
                    }
                }
            }
            else if (currentScreen == 7) //high scores
            {

            }
            else //regular menu screen
            {
                //move
                if (screenArray[currentScreen].numOptions > 1)
                {
                    if (keyset.downButton && !keyset.prevDownButton || keyset.stickDirection == 2 && keyset.prevStickDirection == -1)
                    {
                        if (selectedOption < screenArray[currentScreen].numSelectableOptions - 1) selectedOption++;
                        else selectedOption = 0;

                        audioManager.playSound(1, 1000, 0);
                    }
                    if (keyset.upButton && !keyset.prevUpButton || keyset.stickDirection == 0 && keyset.prevStickDirection == -1)
                    {
                        if (selectedOption > 0) selectedOption--;
                        else selectedOption = screenArray[currentScreen].numSelectableOptions - 1;

                        audioManager.playSound(1, 1000, 0);
                    }
                }

                //Change value
                if (screenArray[currentScreen].optionArray[selectedOption].type == 1)
                {
                    if (keyset.leftButton && !keyset.prevLeftButton || keyset.stickDirection == 3 && keyset.prevStickDirection == -1)
                    {
                        if (screenArray[currentScreen].optionArray[selectedOption].currentValue > 0) screenArray[currentScreen].optionArray[selectedOption].currentValue--;
                        else screenArray[currentScreen].optionArray[selectedOption].currentValue = screenArray[currentScreen].optionArray[selectedOption].numValues - 1;

                        audioManager.playSound(1, 1000, 0);
                    }
                    if (keyset.rightButton && !keyset.prevRightButton || keyset.stickDirection == 1 && keyset.prevStickDirection == -1)
                    {
                        if (screenArray[currentScreen].optionArray[selectedOption].currentValue < screenArray[currentScreen].optionArray[selectedOption].numValues - 1) screenArray[currentScreen].optionArray[selectedOption].currentValue++;
                        else screenArray[currentScreen].optionArray[selectedOption].currentValue = 0;

                        audioManager.playSound(1, 1000, 0);
                    }
                }

                //Select option
                if (keyset.L1Button && !keyset.prevL1Button)
                {

                    int optionID = screenArray[currentScreen].optionArray[selectedOption].optionID;

                    switch (optionID)
                    {
                        case 0: //Press start
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 1;
                            selectedOption = 0;
                            break;
                        case 1: //new game
                            audioManager.playInstance(0, -1);
                            audioManager.playSound(2, 1000, 0);
                            enemyManager.stage = 0;
                            background.drawBackground = true;
                            background.drawClouds = true;
                            background.drawFlatColor = false;
                            return State.game;
                        case 2: //high scores
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 7;
                            break;
                        case 3: //free mode
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 9;
                            selectedOption = 0;
                            break;
                        case 4: //Options
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 2;
                            selectedOption = 3;
                            break;
                        case 5: //Exit game
                            return State.exit;
                        case 6: //Credits
                            break;
                        case 8: //Delete scores
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 6;
                            selectedOption = 1;
                            break;
                        case 10: //play (prueba)
                            audioManager.playSound(2, 1000, 0);
                            enemyManager.stage = screenArray[currentScreen].optionArray[0].currentValue;
                            switch (screenArray[currentScreen].optionArray[1].currentValue)
                            {
                                case 0:
                                    background.drawBackground = true;
                                    background.drawClouds = true;
                                    background.drawFlatColor = false;
                                    break;
                                case 1:
                                    background.drawBackground = false;
                                    background.drawClouds = true;
                                    background.drawFlatColor = true;
                                    break;
                                case 2:
                                    background.drawBackground = false;
                                    background.drawClouds = false;
                                    background.drawFlatColor = true;
                                    break;
                            }
                            return State.game;
                        case 17: //delete (yes)
                            audioManager.playSound(2, 1000, 0);
                            scoreManager.resetTable();
                            currentScreen = 2;
                            selectedOption = 3;
                            break;
                        case 18: //delete (no)
                            audioManager.playSound(2, 1000, 0);
                            currentScreen = 2;
                            selectedOption = 3;
                            break;
                        default:
                            break;
                    }
                }
            }

            //cancel
            if (keyset.XButton & !keyset.prevXButton)
            {
                if (screenArray[currentScreen].prevScreenID != -1)
                {
                    audioManager.playSound(2, 1000, 0);
                    selectedOption = screenArray[currentScreen].prevSelectedOption;
                    currentScreen = screenArray[currentScreen].prevScreenID;
                }             
            }

            return State.mainMenu;
        }
        public void draw(GameTime gameTime, SpriteBatch spriteBatch, ScoreManager scoreManager)
        {
            Vector2 drawPos = new Vector2(0, 0);
            Point kanji;

            //Draw menu background

            pos.X = 0;
            pos.Y = 0;

            for (int i = 0; i < 20; i++)
            {
                drawPos.X = pos.X + i * 16;

                for (int j = 0; j < 15; j++)
                {

                    drawPos.Y = pos.Y + j * 16;

                    spriteBatch.Draw(backgroundImage,
                        drawPos,
                        new Rectangle(0, 0, 16, 16),
                        Color.White,
                        0,
                        Vector2.Zero,
                        1f,
                        SpriteEffects.None,
                        0.1F);
                }
            }

            if (currentScreen == 8) //input new score screen.
            {
                drawPos.Y = 32;
                for (int i = 0; i < scoreManager.highScores.Length; i++)
                {

                    //draw score pos

                    drawPos.X = 32;
                    if (i == scoreManager.newScorePos)
                    {
                        spriteBatch.Draw(kanjiImageBlack,
                            drawPos,
                            new Rectangle(i * 12, 8 * 12, 12, 12),
                            Color.White,
                            0,
                            Vector2.Zero,
                            1f,
                            SpriteEffects.None,
                            0.0F);
                    }
                    else
                    {
                        spriteBatch.Draw(kanjiImage,
                            drawPos,
                            new Rectangle(i * 12, 8 * 12, 12, 12),
                            Color.White,
                            0,
                            Vector2.Zero,
                            1f,
                            SpriteEffects.None,
                            0.0F);
                    }

                    //Draw name

                    drawPos.X += 18;
                    drawPos.X += 8;
                    if (i == scoreManager.newScorePos)
                    {
                        for (int c = 0; c < 3; c++)
                        {
                            spriteBatch.Draw(kanjiImageBlack,
                                drawPos,
                                new Rectangle(scoreManager.highScores[i].name[c] * 12, 0, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                                drawPos.X += 18;
                        }
                    }
                    else
                    {
                        for (int c = 0; c < 3; c++)
                        {
                            spriteBatch.Draw(kanjiImage,
                                drawPos,
                                new Rectangle(scoreManager.highScores[i].name[c] * 12, 0, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                                drawPos.X += 18;
                        }
                    }

                    //Draw score

                    scoreManager.currentScore.ToString();
                    int scoreAux = scoreManager.highScores[i].score;
                    int[] score = new int[7];
                    for (int j = 6; j >= 0; j--)
                    {
                        if (scoreAux > 0) { score[j] = scoreAux % 10; scoreAux = (int)(scoreAux / 10); }
                        else score[j] = 255;
                    }

                    drawPos.X += 18;
                    if (i == scoreManager.newScorePos)
                    {
                        for (int c = 1; c < 7; c++)
                        {
                            spriteBatch.Draw(kanjiImageBlack,
                                drawPos,
                                new Rectangle(score[c] * 12, 8 * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                                drawPos.X += 18;
                        }
                    }
                    else
                    {
                        for (int c = 1; c < 7; c++)
                        {
                            spriteBatch.Draw(kanjiImage,
                                drawPos,
                                new Rectangle(score[c] * 12, 8 * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                                drawPos.X += 18;
                        }
                    }

                    drawPos.Y += 18;
                }
            }
            else if (currentScreen == 7) //high scores screen.
            {
                drawPos.Y = 32;
                for (int i = 0; i < scoreManager.highScores.Length; i++)
                {
                    //draw score pos

                    drawPos.X = 32;
                    spriteBatch.Draw(kanjiImage,
                        drawPos,
                        new Rectangle(i * 12, 8 * 12, 12, 12),
                        Color.White,
                        0,
                        Vector2.Zero,
                        1f,
                        SpriteEffects.None,
                        0.0F);

                    //Draw name

                    drawPos.X += 18;
                    drawPos.X += 8;
                    for (int c = 0; c < 3; c++)
                    {
                        spriteBatch.Draw(kanjiImage,
                            drawPos,
                            new Rectangle(scoreManager.highScores[i].name[c] * 12, 0, 12, 12),
                            Color.White,
                            0,
                            Vector2.Zero,
                            1f,
                            SpriteEffects.None,
                            0.0F);

                        drawPos.X += 18;
                    }

                    //Draw score

                    scoreManager.currentScore.ToString();
                    int scoreAux = scoreManager.highScores[i].score;
                    int[] score = new int[7];
                    for (int j = 6; j >= 0; j--)
                    {
                        if (scoreAux > 0) { score[j] = scoreAux % 10; scoreAux = (int)(scoreAux / 10); }
                        else score[j] = 255;
                    }

                    drawPos.X += 18;
                    for (int c = 1; c < 7; c++)
                    {
                        spriteBatch.Draw(kanjiImage,
                            drawPos,
                            new Rectangle(score[c] * 12, 8 * 12, 12, 12),
                            Color.White,
                            0,
                            Vector2.Zero,
                            1f,
                            SpriteEffects.None,
                            0.0F);
                        drawPos.X += 18;
                    }

                    drawPos.Y += 18;
                }

            }
            else //regular menu screen
            {
                for (int opt = 0; opt < screenArray[currentScreen].numOptions; opt++)
                {
                    //Draw japanese text

                    //if (currentScreen == 3) { if (selectedSlotAux == opt) aux = 108; else aux = 0; }

                    drawPos.Y = screenArray[currentScreen].optionArray[opt].pos.Y;

                    for (int i = 0; i < screenArray[currentScreen].optionArray[opt].kanjiString.Length; i++)
                    {
                        drawPos.X = screenArray[currentScreen].optionArray[opt].pos.X + i * 12; //24 o 32

                        kanji.X = screenArray[currentScreen].optionArray[opt].kanjiString[i] % 30;
                        kanji.Y = screenArray[currentScreen].optionArray[opt].kanjiString[i] / 30;

                        spriteBatch.Draw(kanjiImage,
                            drawPos,
                            new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                            Color.White,
                            0,
                            Vector2.Zero,
                            1f,
                            SpriteEffects.None,
                            0.0F);
                    }

                    //Draw romaji                    
                    if (currentScreen == 3 && opt < 3)
                    {
                        drawPos.X = screenArray[currentScreen].optionArray[opt].pos.X + 16;
                        //Draw save slot names.

                        for (int i = 0; i < screenArray[currentScreen].optionArray[opt].romajiString.Length; i++)
                        {
                            drawPos.Y = screenArray[currentScreen].optionArray[opt].pos.Y + i * 12;

                            kanji.X = screenArray[currentScreen].optionArray[opt].romajiString[i] % 9;
                            kanji.Y = screenArray[currentScreen].optionArray[opt].romajiString[i] / 9;

                            spriteBatch.Draw(kanjiImageBlack,
                                drawPos,
                                new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                        }
                    }
                    else
                    {
                        drawPos.Y = screenArray[currentScreen].optionArray[opt].pos.Y + 16;

                        //Draw romaji text regular options

                        int i;
                        for (i = 0; i < screenArray[currentScreen].optionArray[opt].romajiString.Length; i++)
                        {
                            drawPos.X = screenArray[currentScreen].optionArray[opt].pos.X + i * 12;

                            kanji.X = screenArray[currentScreen].optionArray[opt].romajiString[i] % 30;
                            kanji.Y = screenArray[currentScreen].optionArray[opt].romajiString[i] / 30;

                            spriteBatch.Draw(kanjiImage,
                                drawPos,
                                new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);
                        }

                        if (screenArray[currentScreen].optionArray[opt].type == 1)
                        {
 
                            drawPos.X = screenArray[currentScreen].optionArray[opt].pos.X + i * 12;

                            kanji.X =  27;
                            kanji.Y =  0;

                            spriteBatch.Draw(kanjiImage,
                                drawPos,
                                new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);

                            i++;
                            int j;
                            for (j = 0; j < screenArray[currentScreen].optionArray[opt].values.ElementAt(screenArray[currentScreen].optionArray[opt].currentValue).Length; j++, i++)
                            {
                                drawPos.X = screenArray[currentScreen].optionArray[opt].pos.X + i * 12;

                                kanji.X = screenArray[currentScreen].optionArray[opt].values.ElementAt(screenArray[currentScreen].optionArray[opt].currentValue)[j] % 30;
                                kanji.Y = screenArray[currentScreen].optionArray[opt].values.ElementAt(screenArray[currentScreen].optionArray[opt].currentValue)[j] / 30;

                                spriteBatch.Draw(kanjiImage,
                                    drawPos,
                                    new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                                    Color.White,
                                    0,
                                    Vector2.Zero,
                                    1f,
                                    SpriteEffects.None,
                                    0.0F);
                            }

                            drawPos.X += 12;
                            kanji.X = 28;
                            kanji.Y = 0;
                            spriteBatch.Draw(kanjiImage,
                                drawPos,
                                new Rectangle(kanji.X * 12, kanji.Y * 12, 12, 12),
                                Color.White,
                                0,
                                Vector2.Zero,
                                1f,
                                SpriteEffects.None,
                                0.0F);


                        }

                    }

                }

                //Draw "marked option" icon

                if (currentScreen != 0 && screenArray[currentScreen].numSelectableOptions > 1)
                {
                    drawPos.X = screenArray[currentScreen].optionArray[selectedOption].pos.X - 16;
                    drawPos.Y = screenArray[currentScreen].optionArray[selectedOption].pos.Y + 8;

                    spriteBatch.Draw(kanjiImage,
                        drawPos,
                        new Rectangle(16 * 12, 8 * 12, 12, 12),
                        Color.White,
                        0,
                        Vector2.Zero,
                        1f,
                        SpriteEffects.None,
                        0.0F);

                }

                //Draw game logo

                if (currentScreen == 0)
                {

                    drawPos.X = 128 - 56;
                    drawPos.Y = 56;

                    spriteBatch.Draw(logoImage,
                           drawPos,
                           new Rectangle(0, 0, 108, 36),
                           Color.White,
                           0,
                           Vector2.Zero,
                           1f,
                           SpriteEffects.None,
                           0.0F);
                }
            }
        }
    }
}