﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Game
{
    public struct ScoreInfo
    {
        public int score;
        public int[] name;
    }

    public class ScoreManager
    {
        public ScoreInfo[] highScores;
        public static int numScores = 10;

        public int currentScore = 0;
        public int newScorePos;

        public Texture2D spriteSheet;
       
        public SpriteFont font;
        Vector2 textPos = new Vector2(4, 4);

        public string line;
        public string[] valueString;
        public StreamReader sr;
        public StreamWriter sw;

        public ScoreManager()
        {
            highScores = new ScoreInfo[numScores];
            loadScores();
        }

        public void resetCurrentScore()
        {
            currentScore = 0;
        }

        public void addPoints(int points)
        {
            currentScore += points;
        }

        public void insertHighScore()
        {
            for (newScorePos = numScores - 1; newScorePos >= 0 && currentScore > highScores[newScorePos].score; newScorePos--) { }
            newScorePos++;

            for (int i = numScores - 1; i > newScorePos; i--)
            {
                highScores[i].name[0] = highScores[i - 1].name[0];
                highScores[i].name[1] = highScores[i - 1].name[1];
                highScores[i].name[2] = highScores[i - 1].name[2];

                highScores[i].score = highScores[i - 1].score;
            }

            highScores[newScorePos].name[0] = 0;
            highScores[newScorePos].name[1] = 255;
            highScores[newScorePos].name[2] = 255;

            highScores[newScorePos].score = currentScore;
            currentScore = 0;
        }

        public bool checkNewHighScore()
        {
            if (currentScore > highScores[numScores - 1].score) return true;
            else 
            { 
                currentScore = 0; 
                return false;
            }
        }

        public void saveScores()
        {
            sw = new StreamWriter("highScores.txt");

            for (int i = 0; i < numScores; i++)
            {
                for (int j = 0; j < 3; j++) sw.Write(highScores[i].name[j] + " ");
                sw.WriteLine();
                sw.WriteLine(highScores[i].score);
            }

            sw.Close();

            resetCurrentScore();
        }

        public void loadScores()
        {
            sr = new StreamReader("highScores.txt");
            for (int i = 0; i < numScores; i++)
            {
                highScores[i].name = new int[3];
                line = sr.ReadLine();
                valueString = line.Split(new char[] { ' ', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                highScores[i].name[0] = int.Parse(valueString[0]);
                highScores[i].name[1] = int.Parse(valueString[1]);
                highScores[i].name[2] = int.Parse(valueString[2]);

                line = sr.ReadLine();
                valueString = line.Split(new char[] { ' ', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                highScores[i].score = int.Parse(valueString[0]);
            }
            sr.Close();
        }

        public void resetTable()
        {
            sr = new StreamReader("highScoresOriginal.txt");
            sw = new StreamWriter("highScores.txt");

            while ((line = sr.ReadLine()) != null)
            {
                sw.WriteLine(line);
            }

            sr.Close();
            sw.Close();

            loadScores();
        }
        

        public void draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.DrawString(font, "" + currentScore, textPos, Color.Black);
        }
    }
}
